;**************************************************************************
;*                        AMS Display Driver                              *
;*                    AS1100 / AS1106 / AS1107                            *
;*                         MSSP SPI Version                               *
;**************************************************************************
;**************************************************************************
;*                                                                        *
;*                      v 1.09 - 22.08.2015                               *
;*                                                                        *
;**************************************************************************
;* Hardw.: PIC16F1823 assumed                                             *
;**************************************************************************


;==========================================================================
;       AMS to PIC Connection Definition
;==========================================================================
; connection of the AMS chip to PIC:
;--------------------------------------------
; AMS			Microcontroller
; ------		---------------
; LOAD/CSN (pin 12)	RA5
; CLK      (pin 13)	RC0
; DIN      (pin  1)	RA4

;definition of the connections
#define		AMS_LOAD	PORTA,D'005'

;definition of the corresponding TRISx / ANSLx register
#define		AMS_TRIS1	TRISA,D'005'
#define		AMS_ANSEL1	ANSELA,D'005'
;==========================================================================
;       Variable Definition
;==========================================================================
ams_v_dig0		EQU	H'40'
ams_v_dig1		EQU	H'41'
ams_v_dig2		EQU	H'42'
ams_v_dig3		EQU	H'43'
ams_v_dig4		EQU	H'44'
ams_v_dig5		EQU	H'45'
ams_v_dig6		EQU	H'46'
ams_v_dig7		EQU	H'47'

ams_v_MSB		EQU	H'48'
ams_v_LSB		EQU	H'49'

ams_err_flag		EQU	H'4A'

ams_v_value		EQU	H'4B'
ams_v_tmp		EQU	H'4C'
ams_v_tmp2		EQU	H'4D'
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
AMS_test	movlw	B'11111111'		;register XFh --> display test
		banksel	ams_v_MSB
		movwf	ams_v_MSB
		movlw	B'11111111'		;last bit = 1 --> display test
		movwf	ams_v_LSB
		call	AMS_send_data
		return

AMS_test_off	movlw	B'11111111'		;register XFh --> display test
		banksel	ams_v_MSB
		movwf	ams_v_MSB
		movlw	B'11111110'		;last bit = 1 --> mormal operation
		movwf	ams_v_LSB
		call	AMS_send_data
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
AMS_init	nop
;initialize the required I/Os of the PIC and the SPI mode
		banksel	APFCON			;select appropriate memory bank
		bsf	APFCON,SDOSEL		;RA4 = SDO
		banksel	TRISA			;select appropriate memory bank
		bcf	AMS_TRIS1		;used as CSN/LOAD
		bcf	TRISA,D'004'		;used as SDO
		bcf	TRISC,D'000'		;used as CLK

		#ifdef ANSELA
		banksel	ANSELA			;select appropriate memory bank
		bcf	AMS_ANSEL1
		#endif

		banksel	PORTA			;select Bank 0

		bsf	AMS_LOAD		;LOAD/CSN = 1
;----------------------------------------------------------------------------
;set-up SPI Master Mode of the MSSP
		banksel	SSP1CON1		;select appropriate memory bank
		movlw	B'00000010'		;see Datasheet Page 260
						;bit 7   = 0    -> N/A
						;bit 6   = 0    -> N/A
						;bit 5   = 1    -> enables MSSP1
						;bit 4   = 0    -> Idle state for clock is a low level [CKP]
						;bit 3-0 = 0010 -> SPI Master mode, clock = FOSC/64
		movwf	SSP1CON1

		clrf	SSP1STAT		;bit 7   = 0 -> Input data sampled at middle of data output time
						;bit 6   = 0 -> Transmit occurs on transition from Idle to active clock state [CKE]

		bsf	SSP1STAT,CKE

		bsf	SSP1CON1,D'005'		;enable SPI on MSSP2

		banksel	ams_v_MSB
;----------------------------------------------------------------------------
;initialize the AMS1107
		movlw	B'11111100'		;register XCh --> Shutdown register
		movwf	ams_v_MSB
		movlw	H'01'			;normal operation
		movwf	ams_v_LSB
		call	AMS_send_data
;
		movlw	H'F9'
		movwf	ams_v_MSB		;decode enable register
		movlw	H'00'
		movwf	ams_v_LSB		;no decode mode
		call	AMS_send_data
;
		movlw	H'FB'
		movwf	ams_v_MSB		;scan limit register
		movlw	H'07'
		movwf	ams_v_LSB		;all 8 digits
		call	AMS_send_data
;
		movlw	H'0F'			;default - maximal intensity
		call	AMS_intensity

		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
AMS_intensity	nop
		banksel	ams_v_MSB
		movwf	ams_v_LSB		;xFh = max / x0h = min
		movlw	H'FA'
		movwf	ams_v_MSB		;intensity register

		call	AMS_send_data
		return		
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
AMS_disp_direct	movlw	H'F1'			;DIG0
		banksel	ams_v_MSB
		movwf	ams_v_MSB
		movf	ams_v_dig0,0
		call	AMS_send_data

		movlw	H'F2'			;DIG1
		movwf	ams_v_MSB
		movf	ams_v_dig1,0
		call	AMS_send_data

		movlw	H'F3'			;DIG2
		movwf	ams_v_MSB
		movf	ams_v_dig2,0
		call	AMS_send_data

		movlw	H'F4'			;DIG3
		movwf	ams_v_MSB
		movf	ams_v_dig3,0
		call	AMS_send_data

		movlw	H'F5'			;DIG4
		movwf	ams_v_MSB
		movf	ams_v_dig4,0
		call	AMS_send_data

		movlw	H'F6'			;DIG5
		movwf	ams_v_MSB
		movf	ams_v_dig5,0
		call	AMS_send_data

		movlw	H'F7'			;DIG6
		movwf	ams_v_MSB
		movf	ams_v_dig6,0
		call	AMS_send_data

		movlw	H'F8'			;DIG7
		movwf	ams_v_MSB
		movf	ams_v_dig7,0
		call	AMS_send_data
		return
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
AMS_disp_code	movlw	H'F1'			;DIG0
		banksel	ams_v_MSB
		movwf	ams_v_MSB
		movf	ams_v_dig0,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F2'			;DIG1
		movwf	ams_v_MSB
		movf	ams_v_dig1,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F3'			;DIG2
		movwf	ams_v_MSB
		movf	ams_v_dig2,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F4'			;DIG3
		movwf	ams_v_MSB
		movf	ams_v_dig3,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F5'			;DIG4
		movwf	ams_v_MSB
		movf	ams_v_dig4,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F6'			;DIG5
		movwf	ams_v_MSB
		movf	ams_v_dig5,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F7'			;DIG6
		movwf	ams_v_MSB
		movf	ams_v_dig6,0
		call	AMS_get_char
		call	AMS_send_data

		movlw	H'F8'			;DIG7
		movwf	ams_v_MSB
		movf	ams_v_dig7,0
		call	AMS_get_char
		call	AMS_send_data
		return
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
AMS_get_char	movwf	ams_v_tmp
		movwf	ams_v_tmp2
		bcf	ams_v_tmp,D'007'
		call	AMS_get_char2

		movf	ams_v_LSB,0
		btfsc	ams_v_tmp2,D'007'
		addlw	D'128'

		movwf	ams_v_LSB

		return
;----------------------------------------------------------------------------
AMS_get_char2	movf	ams_v_tmp,0
		xorlw	D'000'
		btfsc	STATUS,Z
		goto	AMS_chr0

		movf	ams_v_tmp,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	AMS_chr1

		movf	ams_v_tmp,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	AMS_chr2

		movf	ams_v_tmp,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	AMS_chr3

		movf	ams_v_tmp,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	AMS_chr4

		movf	ams_v_tmp,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	AMS_chr5

		movf	ams_v_tmp,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	AMS_chr6

		movf	ams_v_tmp,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	AMS_chr7

		movf	ams_v_tmp,0
		xorlw	D'008'
		btfsc	STATUS,Z
		goto	AMS_chr8

		movf	ams_v_tmp,0
		xorlw	D'009'
		btfsc	STATUS,Z
		goto	AMS_chr9

		movf	ams_v_tmp,0
		xorlw	D'010'		;eq H'0A'
		btfsc	STATUS,Z
		goto	AMS_chrA

		movf	ams_v_tmp,0
		xorlw	D'011'		;eq H'0B'
		btfsc	STATUS,Z
		goto	AMS_chrb

		movf	ams_v_tmp,0
		xorlw	D'012'		;eq H'0C'
		btfsc	STATUS,Z
		goto	AMS_chrC

		movf	ams_v_tmp,0
		xorlw	D'013'		;eq H'0D'
		btfsc	STATUS,Z
		goto	AMS_chrd


		movf	ams_v_tmp,0
		xorlw	D'014'		;eq H'0E'
		btfsc	STATUS,Z
		goto	AMS_chrE

		movf	ams_v_tmp,0
		xorlw	D'015'		;eq H'0F'
		btfsc	STATUS,Z
		goto	AMS_chrF

		movf	ams_v_tmp,0
		xorlw	D'016'
		btfsc	STATUS,Z
		goto	AMS_chr_s5

		movf	ams_v_tmp,0
		xorlw	D'017'
		btfsc	STATUS,Z
		goto	AMS_chr_so

		movf	ams_v_tmp,0
		xorlw	D'018'
		btfsc	STATUS,Z
		goto	AMS_chr_stupen

		movf	ams_v_tmp,0
		xorlw	D'019'
		btfsc	STATUS,Z
		goto	AMS_chr_s1

		movf	ams_v_tmp,0
		xorlw	D'020'
		btfsc	STATUS,Z
		goto	AMS_chr_s2

		movf	ams_v_tmp,0
		xorlw	D'021'
		btfsc	STATUS,Z
		goto	AMS_chr_s3

		movf	ams_v_tmp,0
		xorlw	D'022'
		btfsc	STATUS,Z
		goto	AMS_chr_s4

		movf	ams_v_tmp,0
		xorlw	'L'
		btfsc	STATUS,Z
		goto	AMS_chrL

		movf	ams_v_tmp,0
		xorlw	'U'
		btfsc	STATUS,Z
		goto	AMS_chrU

		movf	ams_v_tmp,0
		xorlw	'C'
		btfsc	STATUS,Z
		goto	AMS_chrC

		movf	ams_v_tmp,0
		xorlw	'I'
		btfsc	STATUS,Z
		goto	AMS_chrI

		movf	ams_v_tmp,0
		xorlw	'-'
		btfsc	STATUS,Z
		goto	AMS_chrm

		movf	ams_v_tmp,0
		xorlw	'0'
		btfsc	STATUS,Z
		goto	AMS_chr0

		movf	ams_v_tmp,0
		xorlw	'1'
		btfsc	STATUS,Z
		goto	AMS_chr1

		movf	ams_v_tmp,0
		xorlw	'2'
		btfsc	STATUS,Z
		goto	AMS_chr2

		movf	ams_v_tmp,0
		xorlw	'3'
		btfsc	STATUS,Z
		goto	AMS_chr3

		movf	ams_v_tmp,0
		xorlw	'4'
		btfsc	STATUS,Z
		goto	AMS_chr4

		movf	ams_v_tmp,0
		xorlw	'5'
		btfsc	STATUS,Z
		goto	AMS_chr5

		movf	ams_v_tmp,0
		xorlw	'6'
		btfsc	STATUS,Z
		goto	AMS_chr6

		movf	ams_v_tmp,0
		xorlw	'7'
		btfsc	STATUS,Z
		goto	AMS_chr7

		movf	ams_v_tmp,0
		xorlw	'8'
		btfsc	STATUS,Z
		goto	AMS_chr8

		movf	ams_v_tmp,0
		xorlw	'9'
		btfsc	STATUS,Z
		goto	AMS_chr9

		movf	ams_v_tmp,0
		xorlw	' '
		btfsc	STATUS,Z
		goto	AMS_chr_space

		movf	ams_v_tmp,0
		xorlw	'A'
		btfsc	STATUS,Z
		goto	AMS_chrA

		movf	ams_v_tmp,0
		xorlw	'E'
		btfsc	STATUS,Z
		goto	AMS_chrE

		movf	ams_v_tmp,0
		xorlw	'F'
		btfsc	STATUS,Z
		goto	AMS_chrF

		movf	ams_v_tmp,0
		xorlw	'P'
		btfsc	STATUS,Z
		goto	AMS_chrP

		movf	ams_v_tmp,0
		xorlw	'S'
		btfsc	STATUS,Z
		goto	AMS_chrS

		movf	ams_v_tmp,0
		xorlw	'b'
		btfsc	STATUS,Z
		goto	AMS_chrb

		movf	ams_v_tmp,0
		xorlw	'd'
		btfsc	STATUS,Z
		goto	AMS_chrd

		movf	ams_v_tmp,0
		xorlw	'i'
		btfsc	STATUS,Z
		goto	AMS_chri

		movf	ams_v_tmp,0
		xorlw	'r'
		btfsc	STATUS,Z
		goto	AMS_chrr

		movf	ams_v_tmp,0
		xorlw	't'
		btfsc	STATUS,Z
		goto	AMS_chrt

		movf	ams_v_tmp,0
		xorlw	'o'
		btfsc	STATUS,Z
		goto	AMS_chro

		goto	AMS_chr_err
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
AMS_chr0	movlw	B'01111110'
		movwf	ams_v_LSB
		return

AMS_chr1	movlw	B'00110000'
		movwf	ams_v_LSB
		return

AMS_chr2	movlw	B'01101101'
		movwf	ams_v_LSB
		return

AMS_chr3	movlw	B'01111001'
		movwf	ams_v_LSB
		return

AMS_chr4	movlw	B'00110011'
		movwf	ams_v_LSB
		return

AMS_chr5	movlw	B'01011011'
		movwf	ams_v_LSB
		return

AMS_chr6	movlw	B'01011111'
		movwf	ams_v_LSB
		return

AMS_chr7	movlw	B'01110000'
		movwf	ams_v_LSB
		return

AMS_chr8	movlw	B'01111111'
		movwf	ams_v_LSB
		return

AMS_chr9	movlw	B'01111011'
		movwf	ams_v_LSB
		return

AMS_chrm	movlw	B'00000001'		;minus
		movwf	ams_v_LSB
		return

AMS_chr_space	movlw	B'00000000'
		movwf	ams_v_LSB
		return

AMS_chr_s1	movlw	B'01000000'
		movwf	ams_v_LSB
		return

AMS_chr_s2	movlw	B'00000001'
		movwf	ams_v_LSB
		return

AMS_chr_s3	movlw	B'00001000'
		movwf	ams_v_LSB
		return

AMS_chr_s4	movlw	B'00010010'
		movwf	ams_v_LSB
		return

AMS_chr_s5	movlw	B'00100000'
		movwf	ams_v_LSB
		return

AMS_chr_so	movlw	B'01100011'
		movwf	ams_v_LSB
		return

AMS_chrL	movlw	B'00001110'
		movwf	ams_v_LSB
		return

AMS_chrU	movlw	B'00111110'
		movwf	ams_v_LSB
		return

AMS_chrA	movlw	B'01110111'
		movwf	ams_v_LSB
		return

AMS_chrC	movlw	B'01001110'
		movwf	ams_v_LSB
		return

AMS_chrI	movlw	B'00110000'
		movwf	ams_v_LSB
		return

AMS_chrE	movlw	B'01001111'
		movwf	ams_v_LSB
		return

AMS_chrF	movlw	B'01000111'
		movwf	ams_v_LSB
		return

AMS_chrP	movlw	B'01100111'
		movwf	ams_v_LSB
		return

AMS_chrS	movlw	B'01011011'
		movwf	ams_v_LSB
		return

AMS_chrb	movlw	B'00011111'
		movwf	ams_v_LSB
		return

AMS_chrd	movlw	B'00111101'
		movwf	ams_v_LSB
		return

AMS_chri	movlw	B'00010000'
		movwf	ams_v_LSB
		return

AMS_chrr	movlw	B'00000101'
		movwf	ams_v_LSB
		return

AMS_chrt	movlw	B'00001111'
		movwf	ams_v_LSB
		return

AMS_chro	movlw	B'00011101'
		movwf	ams_v_LSB
		return

AMS_chr_stupen	movlw	B'01100011'
		movwf	ams_v_LSB
		return

AMS_chr_err	movlw	B'01001000'
		movwf	ams_v_LSB
		bsf	ams_err_flag,D'000'
		return

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
AMS_send_data	nop
		banksel	PORTA

		bcf	AMS_LOAD		;LOAD = 0

		banksel	ams_v_MSB
		movf	ams_v_MSB,0
		movwf	ams_v_value
		call	AMS_spi_send

		movf	ams_v_LSB,0
		movwf	ams_v_value
		call	AMS_spi_send

		banksel	PORTA
		bsf	AMS_LOAD		;LOAD = 1

		banksel	ams_v_MSB
		return
;----------------------------------------------------------------------------
AMS_spi_send	nop
		banksel SSP1CON1
		bcf	SSP1CON1,SSPOV
		bcf	SSP1STAT,BF

		movf	ams_v_value
		movwf	SSP1BUF
AMS_spil2	btfss	SSP1STAT,BF
		goto	AMS_spil2
		banksel	ams_v_MSB
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

